@@ -1,12 +1,11 @@
-use crate::ast;
-use crate::{Parse, Spanned, ToTokens};
+use crate::ast::prelude::*;
 
 /// A `while` loop: `while [expr] { ... }`.
 ///
 /// # Examples
 ///
-/// ```rust
-/// use rune::{testing, ast};
+/// ```
+/// use rune::{ast, testing};
 ///
 /// testing::roundtrip::<ast::ExprWhile>("while x {}");
 /// testing::roundtrip::<ast::ExprWhile>("'label: while x {}");
@@ -14,6 +13,7 @@ use crate::{Parse, Spanned, ToTokens};
 /// ```
 #[derive(Debug, Clone, PartialEq, Eq, Parse, ToTokens, Spanned)]
 #[rune(parse = "meta_only")]
+#[non_exhaustive]
 pub struct ExprWhile {
     /// The attributes for the `while` loop
     #[rune(iter, meta)]
@@ -24,7 +24,7 @@ pub struct ExprWhile {
     /// The `while` keyword.
     pub while_token: T![while],
     /// The name of the binding.
-    pub condition: ast::Condition,
+    pub condition: Box<ast::Condition>,
     /// The body of the while loop.
     pub body: Box<ast::Block>,
 }
