@@ -1,18 +1,14 @@
+mod assert_send;
 mod consts;
 mod custom;
-mod description;
 mod gen;
 mod items;
 mod scopes;
-#[cfg(compiler_v2)]
-mod with_span;
 
+pub(crate) use self::assert_send::AssertSend;
 pub(crate) use self::consts::Consts;
 pub(crate) use self::custom::Custom;
-pub(crate) use self::description::Description;
 pub(crate) use self::gen::Gen;
 pub(crate) use self::items::Items;
 pub(crate) use self::scopes::Scopes;
-pub use self::scopes::{ScopeError, ScopeErrorKind};
-#[cfg(compiler_v2)]
-pub(crate) use self::with_span::{ResultExt, WithSpan};
+pub(crate) use self::scopes::{ScopeError, ScopeErrorKind};
