@@ -1,5 +1,6 @@
+use crate::compile::ModMeta;
 use crate::worker::{Import, WildcardImport};
-use runestick::{CompileMod, SourceId};
+use crate::SourceId;
 use std::path::PathBuf;
 use std::sync::Arc;
 
@@ -13,7 +14,7 @@ pub(crate) enum Task {
         /// The source id of the item being loaded.
         source_id: SourceId,
         /// The item of the file to load.
-        mod_item: Arc<CompileMod>,
+        mod_item: Arc<ModMeta>,
     },
     /// Expand a single import.
     ExpandImport(Import),
